function [cdf,pdf,grid]=bernsteindens(a,n)

%Compute the probability and cumulative density functions of a Bernstein
%copula

%input:
%a=Bernstein copula parameters; it is a vector that contains the stacked
%columns of the A matrix as defined in Sancetta and Satchell (2004),
%excluding the elements from the first and last columns and rows
%n=number of evenly spaced grid points

%output:
%cdf=copula
%pdf=density of copula
%grid=grid of points for which the density is computed

grid=linspace(0,1,n);

M=sqrt(length(a))+1;%order of Bernstein copula
%create A matrix as in Sancetta and Satchell (2004)
A=zeros(M+1,M+1);
A(end,:)=linspace(0,1,M+1);
A(:,end)=A(end,:)';
A(2:end-1,2:end-1)=reshape(a,M-1,M-1);

%create matrix P with polynomials of the grid points for the copula
binoms=zeros(1,M+1);
for i1=0:1:M
    binoms(i1+1)=nchoosek(M,i1);
end
polord=0:1:M;
P=(ones(n,1)*binoms).*((grid'*ones(1,M+1)).^(ones(n,1)*polord)).*(((1-grid')*ones(1,M+1)).^(M-ones(n,1)*polord));
cdf=P*A*P';%copula

%create matrix of B coefficients
B=(A(2:end,2:end)+A(1:end-1,1:end-1)-A(2:end,1:end-1)-A(1:end-1,2:end))*M^2;
%create matrix P with polynomials of the grid points for the density
binoms=zeros(1,M);
for i1=0:1:M-1
    binoms(i1+1)=nchoosek(M-1,i1);
end
polord=0:1:M-1;
P=(ones(n,1)*binoms).*((grid'*ones(1,M)).^(ones(n,1)*polord)).*(((1-grid')*ones(1,M)).^(M-1-ones(n,1)*polord));
pdf=P*B*P';%density