clear
clc

%Monte Carlo that compares the CBRE and the RE estimators. The CBRE
%estimator uses four different grid approximations (25/08/2018)

N=5000;%sample size
nc=10*ones(N/10,1);%clusters: groups of size 5
cumnc=[0;cumsum(nc)];
familysamp='clayton';%true copula
familyest='Clayton';%estimation copula
rho=8;%copula parameter
T=4;%time periods
beta=[-1.5;-1;-.5;0;1;3];%marginal parameters
beta0re=beta;%initial value for the RE estimator
beta0cbre=[beta;rho];

K=1;%number of covariates (on top of time dummies)
reps=1000;%repetitions
NN=[10,10,50,50;10,50,10,50];%different combinations of N1,N2, grid size for the copula
R=20;%grid size for the quadrature algorithm
maxit=15;%maximum number of iterations
tol=10^-2;%maximum tolerance

%predefine some vectors
betahatrel=zeros(T+K+1,reps);
betahatrelc1=zeros(T+K+2,reps);
betahatrelc2=zeros(T+K+2,reps);
betahatrelc3=zeros(T+K+2,reps);
betahatrelc4=zeros(T+K+2,reps);
probsrel=zeros(6,reps);
probsrelc1=zeros(6,reps);
probsrelc2=zeros(6,reps);
probsrelc3=zeros(6,reps);
probsrelc4=zeros(6,reps);
trueprobs=zeros(6,reps);
llrel=zeros(1,reps);
llrelc1=zeros(1,reps);
llrelc2=zeros(1,reps);
llrelc3=zeros(1,reps);
llrelc4=zeros(1,reps);
parfor i0=1:1:reps
    x=[kron(eye(T),ones(N,1)),rand(N*T,K)];%generate covariates vector
    eta=zeros(N,1);%generate individual effects
    for i=1:1:length(nc)
        eta(cumnc(i)+1:cumnc(i+1),1)=sampleeta(familysamp,rho,nc(i))';
    end
    ystar=beta(end)*kron(ones(T,1),eta)+x*beta(1:end-1)+random('logistic',0,1,N*T,1);%latent index
    y=double(ystar>0);%dependent variable
    y0=y;
    x0=x;
    y=reshape(y,N,T);
    x=reshape(x(:,5:end),N,T*K);
    
    %random effects
    [b,Htrel1,llrel(i0)]=bc_re(y,x,beta0re,R,maxit,tol,'logit');%parameter estimates & log-likelihood
    betahatrel(:,i0)=b;
    [jointp,condp]=problogitre(y,x,nc,b,R);%joint and conditional probabilities estimates
    probsrel(:,i0)=mean([jointp(:,1:4),condp(:,1:2)],1)';
    
    %copula-based random effects
    N1=NN(1,1);
    N2=NN(2,1);
    [b,~,llrelc1(i0)]=bc_cbre(y,x,nc,beta0cbre,maxit,tol,'logit',familyest,N1,N2);%parameter estimates & log-likelihood
    betahatrelc1(:,i0)=b;
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest,'logit',N1,N2);%joint and conditional probabilities estimates
    probsrelc1(:,i0)=mean([jointp(:,1:4),condp(:,1:2)],1)';
    
    N1=NN(1,2);
    N2=NN(2,2);
    [b,~,llrelc2(i0)]=bc_cbre(y,x,nc,beta0cbre,maxit,tol,'logit',familyest,N1,N2);%parameter estimates & log-likelihood
    betahatrelc2(:,i0)=b;
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest,'logit',N1,N2);%joint and conditional probabilities estimates
    probsrelc2(:,i0)=mean([jointp(:,1:4),condp(:,1:2)],1)';
    
    N1=NN(1,3);
    N2=NN(2,3);
    [b,~,llrelc3(i0)]=bc_cbre(y,x,nc,beta0cbre,maxit,tol,'logit',familyest,N1,N2);%parameter estimates & log-likelihood
    betahatrelc3(:,i0)=b;
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest,'logit',N1,N2);%joint and conditional probabilities estimates
    probsrelc3(:,i0)=mean([jointp(:,1:4),condp(:,1:2)],1)';
    
    N1=NN(1,4);
    N2=NN(2,4);
    [b,~,llrelc4(i0)]=bc_cbre(y,x,nc,beta0cbre,maxit,tol,'logit',familyest,N1,N2);%parameter estimates & log-likelihood
    betahatrelc4(:,i0)=b;
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest,'logit',N1,N2);%joint and conditional probabilities estimates
    probsrelc4(:,i0)=mean([jointp(:,1:4),condp(:,1:2)],1)';
        
    %true approximated probabilities
    [jointp,djointp,condp,dcondp]=jointprobs(y,x,nc,beta0cbre,familyest,'logit',200,200);
    trueprobs(:,i0)=mean([jointp(:,1:4),condp(:,1:2)],1)';
    disp(['Monte Carlo repetition: ' num2str(i0)]);
end

probsrelc1=[probsrelc1;sum(probsrelc1([1,3],:),1);sum(probsrelc1(1:2,:),1)];
probsrelc2=[probsrelc2;sum(probsrelc2([1,3],:),1);sum(probsrelc2(1:2,:),1)];
probsrelc3=[probsrelc3;sum(probsrelc3([1,3],:),1);sum(probsrelc3(1:2,:),1)];
probsrelc4=[probsrelc4;sum(probsrelc4([1,3],:),1);sum(probsrelc4(1:2,:),1)];
probsrel=[probsrel;sum(probsrel([1,3],:),1);sum(probsrel(1:2,:),1)];
trueprobs=[trueprobs;sum(trueprobs([1,3],:),1);sum(trueprobs(1:2,:),1)];

%table with parameter estimates (mean and standard deviation across
%repetitions) and log-likelihood
table1=zeros(2*(T+K+2)+1,5);

for i1=1:1:T+K+1
    table1(2*(i1-1)+1:2*i1,1)=[mean(betahatrelc1(i1,:)),sqrt(var(betahatrelc1(i1,:)))];
    table1(2*(i1-1)+1:2*i1,2)=[mean(betahatrelc2(i1,:)),sqrt(var(betahatrelc2(i1,:)))];
    table1(2*(i1-1)+1:2*i1,3)=[mean(betahatrelc3(i1,:)),sqrt(var(betahatrelc3(i1,:)))];
    table1(2*(i1-1)+1:2*i1,4)=[mean(betahatrelc4(i1,:)),sqrt(var(betahatrelc4(i1,:)))];
    table1(2*(i1-1)+1:2*i1,5)=[mean(betahatrel(i1,:)),sqrt(var(betahatrel(i1,:)))];
end
table1(2*(T+K+1)+1:2*(T+K+2),1)=[mean(betahatrelc1(end,:)),sqrt(var(betahatrelc1(end,:)))];
table1(2*(T+K+1)+1:2*(T+K+2),2)=[mean(betahatrelc2(end,:)),sqrt(var(betahatrelc2(end,:)))];
table1(2*(T+K+1)+1:2*(T+K+2),3)=[mean(betahatrelc3(end,:)),sqrt(var(betahatrelc3(end,:)))];
table1(2*(T+K+1)+1:2*(T+K+2),4)=[mean(betahatrelc4(end,:)),sqrt(var(betahatrelc4(end,:)))];
table1(end,:)=[mean(llrelc1),mean(llrelc2),mean(llrelc3),mean(llrelc4),mean(llrel)];

%table with estimated probabilities (mean and standard deviation across
%repetitions)
table2=zeros(2*8,6);
for i1=1:1:8
    table2((i1-1)*2+1:i1*2,1)=[mean(probsrelc1(i1,:)),sqrt(var(probsrelc1(i1,:)))];
    table2((i1-1)*2+1:i1*2,2)=[mean(probsrelc2(i1,:)),sqrt(var(probsrelc2(i1,:)))];
    table2((i1-1)*2+1:i1*2,3)=[mean(probsrelc3(i1,:)),sqrt(var(probsrelc3(i1,:)))];
    table2((i1-1)*2+1:i1*2,4)=[mean(probsrelc4(i1,:)),sqrt(var(probsrelc4(i1,:)))];
    table2((i1-1)*2+1:i1*2,5)=[mean(probsrel(i1,:)),sqrt(var(probsrel(i1,:)))];
    table2((i1-1)*2+1:i1*2,6)=[mean(trueprobs(i1,:)),0];
end

if N==1000
    save montecarlo3_1.mat betahatrelc1 betahatrelc2 betahatrelc3 betahatrelc4 betahatrel probsrel probsrelc1 probsrelc2 probsrelc3 probsrelc4 trueprobs table1 table2
elseif N==2000
    save montecarlo3_2.mat betahatrelc1 betahatrelc2 betahatrelc3 betahatrelc4 betahatrel probsrel probsrelc1 probsrelc2 probsrelc3 probsrelc4 trueprobs table1 table2
elseif N==5000
    save montecarlo3_3.mat betahatrelc1 betahatrelc2 betahatrelc3 betahatrelc4 betahatrel probsrel probsrelc1 probsrelc2 probsrelc3 probsrelc4 trueprobs table1 table2
end