clear
clc

%Monte Carlo that compares the estimated asymptotic variance of the RE and
%CBRE estimators in two cases: dependent and independent individual effects
%(25/08/2018)

N=2000;%sample size
nc=10*ones(N/10,1);%clusters: groups of size 5
cumnc=[0;cumsum(nc)];
familysamp='clayton';%true copula
familyest='Clayton';%estimation copula 1
epsilon='logit';
rho=8;%copula parameter
T=4;%
beta=[-1.5;-1;-.5;0;1;3];%marginal parameters
beta0=beta;%initial value for the RE estimator

K=1;%number of covariates (on top of time dummies)
reps=1000;%repetitions
N1=50;% grid size of the copula
N2=50;
R=20;%grid size for the quadrature algorithm
maxit=15;%maximum number of iterations
tol=10^-2;%maximum tolerance

%predefine some vectors
betahatrel1=zeros(T+K+1,reps);
betahatrel2=zeros(T+K+1,reps);
betahatrelc1=zeros((T+K+2)*1,reps);
betahatrelc2=zeros((T+K+2)*1,reps);
serel1=zeros(T+K+1,reps);
serel2=zeros(T+K+1,reps);
serelc1=zeros((T+K+2)*1,reps);
serelc2=zeros((T+K+2)*1,reps);
parfor i0=1:1:reps
    x=[kron(eye(T),ones(N,1)),rand(N*T,K)];%generate covariates vector
    
    %within cluster dependence case
    eta=zeros(N,1);%generate individual effects
    for i=1:1:length(nc)
        eta(cumnc(i)+1:cumnc(i+1),1)=sampleeta(familysamp,rho,nc(i))';
    end
    ystar=beta(end)*kron(ones(T,1),eta)+x*beta(1:end-1)+random('logistic',0,1,N*T,1);%latent index
    y=double(ystar>0);%dependent variable
    y0=y;
    x0=x;
    y=reshape(y,N,T);
    x=reshape(x(:,5:end),N,T*K);

    %random effects
    [b,~,~]=bc_re(y,x,beta0,R,maxit,tol,epsilon);
    betahatrel1(:,i0)=b;
    Htrel1=bc_re_var(y,x,nv,b,R,epsilon);
    serel1(:,i0)=sqrt(diag(Htrel1));
    
    %copula-based random effects
    [b,Htrelc1,~]=bc_cbre(y,x,nc,[betahatrel1(:,i0);rho],maxit,tol,epsilon,familyest,N1,N2);
    betahatrelc1(:,i0)=b;
    serelc1(:,i0)=sqrt(diag(inv(Htrelc1)));
    
    %independence case
    x=x0;
    eta=randn(N,1);%generate individual effects
    ystar=beta(end)*kron(ones(T,1),eta)+x*beta(1:end-1)+random('logistic',0,1,N*T,1);%latent index
    y=double(ystar>0);%dependent variable
    y0=y;
    x0=x;
    y=reshape(y,N,T);
    x=reshape(x(:,5:end),N,T*K);

    %random effects
    [b,~,~]=bc_re(y,x,beta0,R,maxit,tol,epsilon);
    betahatrel2(:,i0)=b;
    Htrel2=bc_re_var(y,x,nv,b,R,epsilon);
    serel2(:,i0)=sqrt(diag(Htrel2));
    
    %copula-based random effects
    rhoinit=.001;
    [b,Htrelc2,~]=bc_cbre(y,x,nc,[betahatrel2(:,i0);rhoinit],maxit,tol,epsilon,familyest,N1,N2);
    betahatrelc2(:,i0)=b;
    serelc2(:,i0)=sqrt(diag(inv(Htrelc2)));
    disp(['Monte Carlo repetition: ' num2str(i0)]);
end

%tables with mean estimated standard errors and standard deviation of the
%estimator across repetitions for the RE and CBRE estimators. table1 for
%the dependence case, table2 for the independence case
table1=zeros(T+K+2,4);
table2=zeros(T+K+2,4);
for i1=1:1:T+K+1
    table1(i1,3)=mean(serel1(i1,:));
    table1(i1,4)=sqrt(var(betahatrel1(i1,:)));
    table2(i1,3)=mean(serel2(i1,:));
    table2(i1,4)=sqrt(var(betahatrel2(i1,:)));
end
for i1=1:1:T+K+2
    table1(i1,1)=mean(serelc1(i1,:));
    table1(i1,2)=sqrt(var(betahatrelc1(i1,:)));
    table2(i1,1)=mean(serelc2(i1,:));
    table2(i1,2)=sqrt(var(betahatrelc2(i1,:)));
end


if N==1000
    save montecarlo13_1.mat betahatrel1 betahatrel2 betahatrelc1 betahatrelc2... 
        serel1 serel2 serelc1 serelc2 table1 table2
elseif N==2000
    save montecarlo13_2.mat betahatrel1 betahatrel2 betahatrelc1 betahatrelc2... 
        serel1 serel2 serelc1 serelc2 table1 table2
elseif N==5000
    save montecarlo13_3.mat betahatrel1 betahatrel2 betahatrelc1 betahatrelc2... 
        serel1 serel2 serelc1 serelc2 table1 table2
end