clear
clc

%Monte Carlo that compares the estimation of APE, conditional and joint
%probabilities by the RE estimator, and the CBRE estimator both correctly
%and incorrectly specified, in two cases: dependent and independent
%individual effects (25/08/2018)

N=2000;%sample size
nc=10*ones(N/10,1);%clusters: groups of size 5
cumnc=[0;cumsum(nc)];
familysamp='clayton';%true copula
familyest1='Clayton';%estimation copula 1
familyest2='Gaussian';%estimation copula 2
epsilon1='logit';
epsilon2='probit';
rho=8;%copula parameter
T=4;%
beta=[-1.5;-1;-.5;0;1;3];%marginal parameters
alpha=.05;%Size of test
beta0=beta;%initial value for the RE estimator

K=1;%number of covariates (on top of time dummies)
reps=1000;%repetitions
N1=50;% grid size of the copula
N2=50;
R=20;%grid size for the quadrature algorithm
maxit=15;%maximum number of iterations
tol=10^-2;%maximum tolerance

%predefine some vectors
betahatrel1=zeros(T+K+1,reps);
betahatrel2=zeros(T+K+1,reps);
betahatrelc1=zeros((T+K+2)*1,reps);
betahatrelc2=zeros((T+K+2)*1,reps);
betahatrelc3=zeros((T+K+2)*1,reps);
betahatrelc4=zeros((T+K+2)*1,reps);
betahatrelc5=zeros((T+K+2)*1,reps);
betahatrelc6=zeros((T+K+2)*1,reps);
llrel1=zeros(1,reps);
llrel2=zeros(1,reps);
llrelc1=zeros(1,reps);
llrelc2=zeros(1,reps);
llrelc3=zeros(1,reps);
llrelc4=zeros(1,reps);
llrelc5=zeros(1,reps);
llrelc6=zeros(1,reps);
apere1=zeros(K,reps);
apere2=zeros(K,reps);
aperec1=zeros(K,reps);
aperec2=zeros(K,reps);
aperec3=zeros(K,reps);
aperec4=zeros(K,reps);
aperec5=zeros(K,reps);
aperec6=zeros(K,reps);
probsre1=zeros(3,reps);
probsre2=zeros(3,reps);
probsrec1=zeros(3,reps);
probsrec2=zeros(3,reps);
probsrec3=zeros(3,reps);
probsrec4=zeros(3,reps);
probsrec5=zeros(3,reps);
probsrec6=zeros(3,reps);
trueape1=zeros(K,reps);
trueape2=zeros(K,reps);
trueprobs1=zeros(3,reps);
trueprobs2=zeros(3,reps);
test1=zeros(1,reps);
test2=zeros(1,reps);
test3=zeros(1,reps);
test4=zeros(1,reps);
test5=zeros(1,reps);
test6=zeros(1,reps);
parfor i0=1:1:reps
    x=[kron(eye(T),ones(N,1)),rand(N*T,K)];%generate covariates vector
    
    %within cluster dependence case
    eta=zeros(N,1);%generate individual effects
    for i=1:1:length(nc)
        eta(cumnc(i)+1:cumnc(i+1),1)=sampleeta(familysamp,rho,nc(i))';
    end
    ystar=beta(end)*kron(ones(T,1),eta)+x*beta(1:end-1)+random('logistic',0,1,N*T,1);%latent index
    y=double(ystar>0);%dependent variable
    y0=y;
    x0=x;
    y=reshape(y,N,T);
    x=reshape(x(:,5:end),N,T*K);

    %random effects
    [b,Htrel1,llrel1(i0)]=bc_re(y,x,beta0,R,maxit,tol,epsilon1);
    betahatrel1(:,i0)=b;
    apere1(:,i0)=binaryreape(y,x,b,R,epsilon1);
    [jointp,condp]=problogitre(y,x,nc,b,R);
    probsre1(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';
    
    %copula-based random effects correctly specified
    [b,Htrelc1,llrelc1(i0)]=bc_cbre(y,x,nc,[betahatrel1(:,i0);rho],maxit,tol,epsilon1,familyest1,N1,N2);
    betahatrelc1(:,i0)=b;
    aperec1(:,i0)=binaryreape(y,x,b,R,epsilon1);
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest1,epsilon1,N1,N2);
    probsrec1(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';    

    %copula-based random effects incorrectly specified #1
    [b,Htrelc2,llrelc2(i0)]=bc_cbre(y,x,nc,[betahatrel1(:,i0);.5],maxit,tol,epsilon1,familyest2,N1,N2);
    betahatrelc2(:,i0)=b;
    aperec2(:,i0)=binaryreape(y,x,b,R,epsilon1);
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest2,epsilon1,N1,N2);
    probsrec2(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';  

    %copula-based random effects incorrectly specified #2  
    [b,Htrelc3,llrelc3(i0)]=bc_cbre(y,x,nc,[betahatrel1(:,i0);rho],maxit,tol,epsilon2,familyest1,N1,N2);
    betahatrelc3(:,i0)=b;
    aperec3(:,i0)=binaryreape(y,x,b,R,epsilon2);
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest1,epsilon2,N1,N2);
    probsrec3(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';    
    
    %test of independence for correct CBRE estimator
    [~,~,test1(i0)]=testind(familyest1,betahatrelc1(:,i0),Htrelc1,alpha);
    
    %Schennach-Wilhelm tests for correct CBRE estimator vs incorrect CBRE
    %estimators
    test2(i0)=testsw(y,x,nc,epsilon1,epsilon1,familyest1,familyest2,betahatrelc1(:,i0),betahatrelc2(:,i0),N1,N2,alpha);
    test3(i0)=testsw(y,x,nc,epsilon1,epsilon2,familyest1,familyest1,betahatrelc1(:,i0),betahatrelc3(:,i0),N1,N2,alpha);
    
    %true approximated probabilities
    trueape1(:,i0)=binaryreape(y,x,beta0,R,epsilon1);
    [jointp,~,condp,~]=jointprobs(y,x,nc,[beta0;rho],familyest1,epsilon2,N1,N2);
    trueprobs1(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';
    
    %independence case
    x=x0;
    eta=randn(N,1);%generate individual effects
    ystar=beta(end)*kron(ones(T,1),eta)+x*beta(1:end-1)+random('logistic',0,1,N*T,1);%latent index
    y=double(ystar>0);%dependent variable
    y0=y;
    x0=x;
    y=reshape(y,N,T);
    x=reshape(x(:,5:end),N,T*K);
    
    %random effects
    [b,Htrel2,llrel2(i0)]=bc_re(y,x,beta0,R,maxit,tol,epsilon1);
    betahatrel2(:,i0)=b;
    apere2(:,i0)=binaryreape(y,x,b,R,epsilon1);
    [jointp,condp]=problogitre(y,x,nc,b,R);
    probsre2(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';    
    
    %copula-based random effects correctly specified
    switch familyest1
        case 'Clayton'
            rhoinit=.001;
        case 'Gumbel'
            rhoinit=1.01;
        case 'Frank'
            rhoinit=.01;
        case 'Ali'
            rhoinit=.01;
        case 'Joe'
            rhoinit=1.01;
    end
    [b,Htrelc4,llrelc4(i0)]=bc_cbre(y,x,nc,[betahatrel2(:,i0);rhoinit],maxit,tol,epsilon1,familyest1,N1,N2);
    betahatrelc4(:,i0)=b;
    aperec4(:,i0)=binaryreape(y,x,b,R,epsilon1);
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest1,epsilon1,N1,N2);
    probsrec4(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';    

    %copula-based random effects incorrectly specified #1
    switch familyest2
        case 'Clayton'
            rhoinit=.001;
        case 'Gumbel'
            rhoinit=1.01;
        case 'Frank'
            rhoinit=.01;
        case 'Ali'
            rhoinit=.01;
        case 'Joe'
            rhoinit=1.01;
    end
    [b,Htrelc5,llrelc5(i0)]=bc_cbre(y,x,nc,[betahatrel2(:,i0);rhoinit],maxit,tol,epsilon1,familyest2,N1,N2);
    betahatrelc5(:,i0)=b;
    aperec5(:,i0)=binaryreape(y,x,b,R,epsilon1);
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest2,epsilon1,N1,N2);
    probsrec5(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';    

    %copula-based random effects incorrectly specified #2
    switch familyest1
        case 'Clayton'
            rhoinit=.001;
        case 'Gumbel'
            rhoinit=1.01;
        case 'Frank'
            rhoinit=.01;
        case 'Ali'
            rhoinit=.01;
        case 'Joe'
            rhoinit=1.01;
    end
    [b,Htrelc6,llrelc6(i0)]=bc_cbre(y,x,nc,[betahatrel2(:,i0);rhoinit],maxit,tol,epsilon2,familyest1,N1,N2);
    betahatrelc6(:,i0)=b;
    aperec6(:,i0)=binaryreape(y,x,b,R,epsilon2);
    [jointp,~,condp,~]=jointprobs(y,x,nc,b,familyest1,epsilon2,N1,N2);
    probsrec6(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';      
    
    %test of independence for correct CBRE estimator
    [~,~,test4(i0)]=testind(familyest1,betahatrelc4(:,i0),Htrelc4,alpha);
    
    %Schennach-Wilhelm tests for correct CBRE estimator vs incorrect CBRE
    %estimators
    test5(i0)=testsw(y,x,nc,epsilon1,epsilon1,familyest1,familyest2,betahatrelc4(:,i0),betahatrelc5(:,i0),N1,N2,alpha);
    test6(i0)=testsw(y,x,nc,epsilon1,epsilon2,familyest1,familyest1,betahatrelc4(:,i0),betahatrelc6(:,i0),N1,N2,alpha);
    
    %true approximated probabilities
    trueape2(:,i0)=binaryreape(y,x,beta0,R,epsilon1);
    [jointp,~,condp,~]=jointprobs(y,x,nc,[beta0;.001],familyest1,epsilon2,N1,N2);
    trueprobs2(:,i0)=mean([jointp(:,1),condp(:,1:2)],1)';

    disp(['Monte Carlo repetition: ' num2str(i0)]);
end

%tables with APE (mean and standard deviation across repetitions), mean
%estimated probabilities (P(y1=1,y2=1), P(y1=1|y2=1), P(y1=1|y2=0)), and
%proportion of times each hypothesis is not rejected for the SW and
%independence tests. table1 for the dependence case, table2 for the
%independence case
table1=[mean(aperec1),mean(aperec2),mean(aperec3),mean(apere1),mean(trueape1);
    sqrt(var(aperec1)),sqrt(var(aperec2)),sqrt(var(aperec3)),sqrt(var(apere1)),0;
    mean(probsrec1,2),mean(probsrec2,2),mean(probsrec3,2),mean(probsre1,2),mean(trueprobs1,2);
    0,sum((test2>.025).*(test2<.975))/reps*100,sum((test3>.025).*(test3<.975))/reps*100,sum(test1>.05)/reps*100,0;
    0,sum(test2<.025)/reps*100,sum(test3<.025)/reps*100,sum(test1<.05)/reps*100,0;
    0,sum(test2>.975)/reps*100,sum(test3>.975)/reps*100,0,0];

table2=[mean(aperec4),mean(aperec5),mean(aperec6),mean(apere2),mean(trueape2);
    sqrt(var(aperec4)),sqrt(var(aperec5)),sqrt(var(aperec6)),sqrt(var(apere2)),0;
    mean(probsrec4,2),mean(probsrec5,2),mean(probsrec6,2),mean(probsre2,2),mean(trueprobs2,2);
    0,sum((test5>.025).*(test5<.975))/reps*100,sum((test6>.025).*(test6<.975))/reps*100,sum(test4>.05)/reps*100,0;
    0,sum(test5<.025)/reps*100,sum(test6<.025)/reps*100,sum(test4<.05)/reps*100,0;
    0,sum(test5>.975)/reps*100,sum(test6>.975)/reps*100,0,0];


if N==1000
    save montecarlo4_1.mat apere1 apere2 aperec1 aperec2 aperec3 aperec4 aperec5 aperec6...
    llrel1 llrel2 llrelc1 llrelc2 llrelc3 llrelc4 llrelc5 llrelc6...
    probsre1 probsre2 probsrec1 probsrec2 probsrec3 probsrec4 probsrec5 probsrec6...
    trueape1 trueape2 trueprobs1 trueprobs2 test1 test2 test3 test4 test5 test6 table1 table2
elseif N==2000
    save montecarlo4_2.mat apere1 apere2 aperec1 aperec2 aperec3 aperec4 aperec5 aperec6...
    llrel1 llrel2 llrelc1 llrelc2 llrelc3 llrelc4 llrelc5 llrelc6...
    probsre1 probsre2 probsrec1 probsrec2 probsrec3 probsrec4 probsrec5 probsrec6...
    trueape1 trueape2 trueprobs1 trueprobs2 test1 test2 test3 test4 test5 test6 table1 table2
elseif N==5000
    save montecarlo4_3.mat apere1 apere2 aperec1 aperec2 aperec3 aperec4 aperec5 aperec6...
    llrel1 llrel2 llrelc1 llrelc2 llrelc3 llrelc4 llrelc5 llrelc6...
    probsre1 probsre2 probsrec1 probsrec2 probsrec3 probsrec4 probsrec5 probsrec6...
    trueape1 trueape2 trueprobs1 trueprobs2 test1 test2 test3 test4 test5 test6 table1 table2
end