function [jointp,condp]=problogitre(y,x,nc,b,R)

%estimator of the joint and conditional probabilities for the logit RE
%estimator
%the probability is calculated for (y1,y2) in each group and period

%input:
%y=dependent variable
%x=covariates (excluding time dummies)
%nc=group sizes
%b=initial value of parameters
%R=grid size of the quadrature

%output:
%jointp=joint probabilities:(1,1),(1,0),(0,1),(0,0)
%condp=conditional probabilities:(1|1),(1|0)

[u,wu]=GaussHermite_2(R);%grid for the Gauss-Hermite quadrature

[~,T]=size(y);%number of periods
[~,K]=size(x);
K=K/T;%number of covariates
cumnc=[0;cumsum(nc)];
C=length(nc);

%predefine some variables
jointp=zeros(C,4*T);
condp=zeros(C,2*T);
for i1=1:1:C
    p2=zeros(T,1);
    p3=zeros(T,1);
    p4=zeros(T,1);
    p5=zeros(T,1);
    xi1=[eye(T),x(cumnc(i1)+1,:)'];%covariates for i=1
    z1=kron(u*b(T+K+1)*sqrt(2),ones(1,T))+ones(R,1)*b(1:T+K)'*xi1';
    h1=exp(z1)./(1+exp(z1));
    xi2=[eye(T),x(cumnc(i1)+2,:)'];%covariates for i=2
    z2=kron(u*b(T+K+1)*sqrt(2),ones(1,T))+ones(R,1)*b(1:T+K)'*xi2';
    h2=exp(z2)./(1+exp(z2));
    
    p1(1,:)=wu'*h1/sqrt(pi);%Probability y_{1t}=1
    p1(2,:)=wu'*h2/sqrt(pi);%Probability y_{2t}=1
    
    %compute joint probabilities
    for i2=1:1:T
        p2(i2,1)=p1(1,i2)*p1(2,i2);
        p3(i2,1)=(1-p1(1,i2))*p1(2,i2);
        p4(i2,1)=p1(1,i2)*(1-p1(2,i2));
        p5(i2,1)=(1-p1(1,i2))*(1-p1(2,i2));
        jointp(i1,(i2-1)*4+1)=p2(i2);%Probability y_{1t}=1, y_{2t}=1
        jointp(i1,(i2-1)*4+2)=p3(i2);%Probability y_{1t}=0, y_{2t}=1
        jointp(i1,(i2-1)*4+3)=p4(i2);%Probability y_{1t}=1, y_{2t}=0
        jointp(i1,(i2-1)*4+4)=p5(i2);%Probability y_{1t}=0, y_{2t}=0
    end
    
    %compute conditional probabilities
    for i2=1:1:T
        condp(:,(i2-1)*2+1)=jointp(:,(i2-1)*4+1)./(jointp(:,(i2-1)*4+1)+jointp(:,(i2-1)*4+2));% Probability y_{1t}=1|y_{2t}=1
        condp(:,(i2-1)*2+2)=jointp(:,(i2-1)*4+3)./(jointp(:,(i2-1)*4+3)+jointp(:,(i2-1)*4+4));% Probability y_{1t}=1|y_{2t}=1
    end
end