function test=testsw(y,x,nc,epsilon1,epsilon2,family1,family2,beta1,beta2,N1,N2,alpha)

%Schennach-Wilhelm test

%input:
%y=dependent variable
%x=covariates (excluding time dummies)
%nc=group sizes
%epsilon={logit,probit}
%family={Gaussian,Clayton,Frank,Gumbel}
%beta=parameters
%N1,N2=number of points for copula grid
%alpha=size of test

%output:
%test=p-value of test

%make sure copula parameter belongs to parameter space
C=length(nc);

[~,z01]=logitcopulare_ll(y,x,nc,beta1,epsilon1,family1,N1,N2);
[~,z02]=logitcopulare_ll(y,x,nc,beta2,epsilon2,family2,N1,N2);

sigmaa=mean((log(z01)-mean(log(z01))).^2);
sigmab=mean((log(z02)-mean(log(z02))).^2);
sigmaab=mean((log(z01)-mean(log(z01))).*(log(z02)-mean(log(z02))));
sigma=sigmaa+sigmab-2*sigmaab;
z=norminv(alpha/2);
delta=sigma/(2*(z-sqrt(4+z^2)));

Cpl=normpdf(z-delta/sqrt(sigma))*(delta*(sigma-2*sigmaa-2*sigmab)/(4*sigma^3));
Csd=2*normpdf(z)*(length(beta1)/sqrt((sigmaa+sigmab)/2));
eps=(Csd/Cpl)^(1/3)*C^(-1/6)*log(log(C))^(1/3);

eps1=zeros(C,1);
eps2=zeros(C,1);
eps1(1:2:end,1)=eps;
eps2(2:2:end,1)=eps;
ll1=log(z01).*(ones(C,1)+eps1);
ll2=log(z02).*(ones(C,1)+eps2);

d=sqrt(C)*(mean(ll2-ll1))/sqrt(sigma*(1+eps)+eps^2/2*(sigmaa+sigmab));
test=normcdf(d);